@@ -3,18 +3,19 @@
 # Copyright (c) CERN, 2021.                 #
 # ######################################### #
 
-from pathlib import Path
-import numpy as np
 from functools import partial
+from pathlib import Path
+
+import numpy as np
 
 import xobjects as xo
+import xtrack as xt
 from xobjects.general import Print
-
 from xobjects.hybrid_class import _build_xofields_dict
-
 from .general import _pkg_root
 from .internal_record import RecordIdentifier, RecordIndex, generate_get_record
 from .particles import Particles
+from .track_flags import c_header_flag_mapping
 
 start_per_part_block = """
     {
@@ -59,8 +60,7 @@ def _handle_per_particle_blocks(sources, local_particle_src):
             strss = ss
 
         strss = strss.replace('/*placeholder_for_local_particle_src*/',
-                                local_particle_src,
-                                )
+                                local_particle_src)
         if '//start_per_particle_block' in strss:
 
             lines = strss.splitlines()
@@ -82,16 +82,20 @@ def _handle_per_particle_blocks(sources, local_particle_src):
     return out
 
 def _generate_track_local_particle_with_transformations(
-                                                element_name,
-                                                allow_rot_and_shift,
-                                                rot_and_shift_from_parent,
-                                                local_particle_function_name):
-
-    source = ('''
-            /*gpufun*/
-            '''
-            f'void {local_particle_function_name}_with_transformations({element_name}Data el, LocalParticle* part0)'
-            '{\n')
+    element_name,
+    allow_rot_and_shift,
+    rot_and_shift_from_parent,
+    local_particle_function_name,
+    isthick,
+    xofields
+):
+    source = (
+        '#include <headers/track.h>\n'
+        '#include <headers/particle_states.h>\n'
+        '#include <beam_elements/elements_src/track_misalignments.h>\n'
+        f'GPUFUN void {local_particle_function_name}_with_transformations({element_name}Data el, LocalParticle* part0)'
+        '{\n'
+    )
 
     if rot_and_shift_from_parent:
         add_to_call = '__parent'
@@ -99,64 +103,130 @@ def _generate_track_local_particle_with_transformations(
         add_to_call = ''
 
     if allow_rot_and_shift:
+        if ('angle' in xofields or
+           (('ThickSlice' in element_name) and ('Bend' in element_name)) or
+           (('ThinSlice' in element_name) and ('Bend' in element_name))
+        ):
+            element_shape = 'curved'
+            misalign_arguments = 'shift_x, shift_y, shift_s, rot_y_rad, rot_x_rad, rot_s_rad_no_frame, anchor, length, angle, rot_s_rad'
+            get_angle = f'double const angle = {element_name}Data_get{add_to_call}_angle(el)'
+            get_rot_s_rad = 'double const rot_s_rad = atan2(_sin_rot_s, _cos_rot_s)'
+        else:
+            element_shape = 'straight'
+            misalign_arguments = 'shift_x, shift_y, shift_s, rot_y_rad, rot_x_rad, rot_s_rad_no_frame, anchor, length'
+            get_angle = ''
+            get_rot_s_rad = ''
+
+        if 'isthick' in xofields:
+            get_length = (
+                f'double const length = {element_name}Data_get{add_to_call}_isthick(el)'
+                f' ? {element_name}Data_get{add_to_call}_length(el)'
+                f' : 0.0'
+            )
+        elif 'length' in xofields and isthick:
+            get_length = f'double const length = {element_name}Data_get{add_to_call}_length(el)'
+        else:
+            get_length = 'double const length = 0.'
 
         source += (
             '    // Transform to local frame\n'
-            # f'    printf("Transform to local frame {element_name}\\n");\n'
             f'double const _sin_rot_s = {element_name}Data_get{add_to_call}__sin_rot_s(el);\n'
             'if (_sin_rot_s > -2.) {\n'
             f'    double const _cos_rot_s = {element_name}Data_get{add_to_call}__cos_rot_s(el);\n'
+            f'    {get_rot_s_rad};\n'
             f'    double const shift_x = {element_name}Data_get{add_to_call}__shift_x(el);\n'
             f'    double const shift_y = {element_name}Data_get{add_to_call}__shift_y(el);\n'
             f'    double const shift_s = {element_name}Data_get{add_to_call}__shift_s(el);\n'
-            '\n'
-            '    if (shift_s != 0.) {\n'
-            '        //start_per_particle_block (part0->part)\n'
-            '            Drift_single_particle(part, shift_s);\n'
-            '        //end_per_particle_block\n'
-            '    }\n'
-            '\n'
-            '    //start_per_particle_block (part0->part)\n'
-            '       LocalParticle_add_to_x(part, -shift_x);\n'
-            '       LocalParticle_add_to_y(part, -shift_y);\n'
-            '    //end_per_particle_block\n'
+            f'    double const rot_x_rad = {element_name}Data_get{add_to_call}__rot_x_rad(el);\n'
+            f'    double const rot_y_rad = {element_name}Data_get{add_to_call}__rot_y_rad(el);\n'
+            f'    double const rot_s_rad_no_frame = {element_name}Data_get{add_to_call}__rot_s_rad_no_frame(el);\n'
+            f'    {get_length};\n'
+            f'    {get_angle};\n'
+            f'    double const anchor = {element_name}Data_get{add_to_call}_rot_shift_anchor(el);\n'
+            '\n')
+
+        if rot_and_shift_from_parent:
+            source += (
+                "if (rot_x_rad != 0 || rot_y_rad != 0) {\n"
+                " //start_per_particle_block (part0->part)\n"
+                "    LocalParticle_set_state(part, XT_INVALID_SLICE_TRANSFORM);\n"
+                " //end_per_particle_block\n"
+                "}\n"
+            )
+            if element_shape == 'curved':
+                source += (
+                    "if (angle != 0 && (shift_x != 0 || shift_y != 0 || shift_s != 0"
+                    "                   || rot_x_rad !=0 || rot_y_rad != 0 || rot_s_rad_no_frame != 0)) {\n"
+                    " //start_per_particle_block (part0->part)\n"
+                    "    LocalParticle_set_state(part, XT_INVALID_CURVED_SLICE_TRANSFORM);\n"
+                    " //end_per_particle_block\n"
+                "}\n"
+            )
+
+        source += (
+            f'    track_misalignment_entry_{element_shape}(part0, {misalign_arguments});'
             '\n'
             '     //start_per_particle_block (part0->part)\n'
             '          SRotation_single_particle(part, _sin_rot_s, _cos_rot_s);\n'
             '     //end_per_particle_block\n'
+            '\n'
+            '    /* Spin tracking is disabled by the synrad compile flag */\n'
+            '    #ifndef XTRACK_MULTIPOLE_NO_SYNRAD\n'
+            '       // Rotate spin\n'
+            '       //start_per_particle_block (part0->part)\n'
+            '           double const spin_x_0 = LocalParticle_get_spin_x(part);\n'
+            '           double const spin_y_0 = LocalParticle_get_spin_y(part);\n'
+            '           if ((spin_x_0 != 0) || (spin_y_0 != 0)){\n'
+            '               double const spin_x_1 = _cos_rot_s*spin_x_0 + _sin_rot_s*spin_y_0;\n'
+            '               double const spin_y_1 = -_sin_rot_s*spin_x_0 + _cos_rot_s*spin_y_0;\n'
+            '               LocalParticle_set_spin_x(part, spin_x_1);\n'
+            '               LocalParticle_set_spin_y(part, spin_y_1);\n'
+            '          }\n'
+            '       //end_per_particle_block\n'
+            '    #endif\n'
+            '\n'
             '}\n'
-        )
-
-    source += (
             f'    {local_particle_function_name}(el, part0);\n'
-    )
-
-    if allow_rot_and_shift:
-        source += (
             '    // Transform back to global frame\n'
-            #f'    printf("Transform to back to global frame {element_name}\\n");\n'
             'if (_sin_rot_s > -2.) {\n'
             f'    double const _cos_rot_s = {element_name}Data_get{add_to_call}__cos_rot_s(el);\n'
+            f'    {get_rot_s_rad};\n'
             f'    double const shift_x = {element_name}Data_get{add_to_call}__shift_x(el);\n'
             f'    double const shift_y = {element_name}Data_get{add_to_call}__shift_y(el);\n'
             f'    double const shift_s = {element_name}Data_get{add_to_call}__shift_s(el);\n'
+            f'    double const rot_x_rad = {element_name}Data_get{add_to_call}__rot_x_rad(el);\n'
+            f'    double const rot_y_rad = {element_name}Data_get{add_to_call}__rot_y_rad(el);\n'
+            f'    double const rot_s_rad_no_frame = {element_name}Data_get{add_to_call}__rot_s_rad_no_frame(el);\n'
+            f'    {get_length};\n'
+            f'    {get_angle};\n'
+            f'    double const anchor = {element_name}Data_get{add_to_call}_rot_shift_anchor(el);\n'
+            '\n'
+            '    /* Spin tracking is disabled by the synrad compile flag */\n'
+            '    #ifndef XTRACK_MULTIPOLE_NO_SYNRAD\n'
+            '       // Rotate spin\n'
+            '       //start_per_particle_block (part0->part)\n'
+            '           double const spin_x_0 = LocalParticle_get_spin_x(part);\n'
+            '           double const spin_y_0 = LocalParticle_get_spin_y(part);\n'
+            '           if ((spin_x_0 != 0) || (spin_y_0 != 0)){\n'
+            '               double const spin_x_1 = _cos_rot_s*spin_x_0 - _sin_rot_s*spin_y_0;\n'
+            '               double const spin_y_1 = _sin_rot_s*spin_x_0 + _cos_rot_s*spin_y_0;\n'
+            '               LocalParticle_set_spin_x(part, spin_x_1);\n'
+            '               LocalParticle_set_spin_y(part, spin_y_1);\n'
+            '          }\n'
+            '       //end_per_particle_block\n'
+            '    #endif\n'
             '\n'
             '    //start_per_particle_block (part0->part)\n'
             '       SRotation_single_particle(part, -_sin_rot_s, _cos_rot_s);\n'
             '    //end_per_particle_block\n'
             '\n'
-            '    //start_per_particle_block (part0->part)\n'
-            '       LocalParticle_add_to_x(part, shift_x);\n'
-            '       LocalParticle_add_to_y(part, shift_y);\n'
-            '    //end_per_particle_block\n'
-            '\n'
-            '    if (shift_s != 0.) {\n'
-            '        //start_per_particle_block (part0->part)\n'
-            '            Drift_single_particle(part, -shift_s);\n'
-            '        //end_per_particle_block\n'
-            '    }\n'
+            f'    track_misalignment_exit_{element_shape}(part0, {misalign_arguments});'
             '}\n'
         )
+    else:
+        source += (
+            f'    {local_particle_function_name}(el, part0);\n'
+        )
     source += '}\n'
     return source
 
@@ -211,6 +281,7 @@ def _generate_per_particle_kernel_from_local_particle_function(
             for (int64_t batch_id = 0; batch_id < num_threads; batch_id++) {                   //only_for_context cpu_openmp
                 LocalParticle lpart;
                 lpart.io_buffer = io_buffer;
+                lpart.track_flags = 0;
                 int64_t part_id = batch_id * chunk_size;                                       //only_for_context cpu_openmp
                 int64_t end_id = (batch_id + 1) * chunk_size;                                  //only_for_context cpu_openmp
                 if (end_id > num_particles_to_track) end_id = num_particles_to_track;          //only_for_context cpu_openmp
@@ -247,15 +318,25 @@ def _generate_per_particle_kernel_from_local_particle_function(
     return source
 
 def _tranformations_active(self):
+    no_shift = self.shift_x == 0 and self.shift_y == 0 and self.shift_s == 0
+    no_rot = (
+        self.rot_x_rad == 0 and self.rot_y_rad == 0 and self.rot_s_rad == 0 and
+        self.rot_s_rad_no_frame == 0
+    )
 
-    if (self.shift_x == 0 and self.shift_y == 0 and self.shift_s == 0
-        and self._sin_rot_s == 0 and self._cos_rot_s >= 0): # means angle is zero
+    if no_shift and no_rot:
         return False
-    elif (self.shift_x == 0 and self.shift_y == 0 and self.shift_s == 0
-          and self._sin_rot_s < -2.):
-        return False
-    else:
-        return True
+
+    return True
+
+
+def _disable_transformations_if_needed(self):
+    if not _tranformations_active(self):
+        self._sin_rot_s = -999.
+    elif self._sin_rot_s < -2.:
+        self._sin_rot_s = 0.
+        self._cos_rot_s = 1.
+
 
 def _rot_s_property(self):
     if self._sin_rot_s < -2.:
@@ -265,50 +346,88 @@ def _rot_s_property(self):
 def _set_rot_s_property_setter(self, value):
     self._sin_rot_s = np.sin(value)
     self._cos_rot_s = np.cos(value)
-    if not _tranformations_active(self):
-        self._sin_rot_s = -999.
-    elif self._sin_rot_s < -2.:
-        self._sin_rot_s = 0.
-        self._cos_rot_s = 1.
+    _disable_transformations_if_needed(self)
 
 def _shiftx_property(self):
     return self._shift_x
 
 def _set_shiftx_property_setter(self, value):
     self._shift_x = value
-    if not _tranformations_active(self):
-        self._sin_rot_s = -999.
-    elif self._sin_rot_s < -2.:
-        self._sin_rot_s = 0.
-        self._cos_rot_s = 1.
+    _disable_transformations_if_needed(self)
 
 def _shifty_property(self):
     return self._shift_y
 
 def _set_shifty_property_setter(self, value):
     self._shift_y = value
-    if not _tranformations_active(self):
-        self._sin_rot_s = -999.
-    elif self._sin_rot_s < -2.:
-        self._sin_rot_s = 0.
-        self._cos_rot_s = 1.
+    _disable_transformations_if_needed(self)
 
 def _shifts_property(self):
     return self._shift_s
 
 def _set_shifts_property_setter(self, value):
     self._shift_s = value
-    if not _tranformations_active(self):
-        self._sin_rot_s = -999.
-    elif self._sin_rot_s < -2.:
-        self._sin_rot_s = 0.
-        self._cos_rot_s = 1.
+    _disable_transformations_if_needed(self)
+
+def _rot_x_property(self):
+    return self._rot_x_rad
+
+def _set_rot_x_property_setter(self, value):
+    self._rot_x_rad = value
+    _disable_transformations_if_needed(self)
+
+def _rot_y_property(self):
+    return self._rot_y_rad
+
+def _set_rot_y_property_setter(self, value):
+    self._rot_y_rad = value
+    _disable_transformations_if_needed(self)
+
+def _rot_s_no_frame_property(self):
+    return self._rot_s_rad_no_frame
+
+def _set_rot_s_no_frame_property_setter(self, value):
+    self._rot_s_rad_no_frame = value
+    _disable_transformations_if_needed(self)
 
 class MetaBeamElement(xo.MetaHybridClass):
 
     def __new__(cls, name, bases, data):
+
         _XoStruct_name = name+'Data'
 
+        data_in = data.copy()
+        data = {}
+
+        for bb in bases:
+            if bb.__name__ == 'HybridClass':
+                continue
+            if bb.__name__ == 'BeamElement':
+                continue
+            for kk, vv in bb.__dict__.items():
+                if kk.startswith('__') or kk in data_in.keys():
+                    continue
+                data[kk] = vv
+
+        # If inheriting _extra_c_sources, remove get_record function
+        if '_extra_c_sources' in data:
+            ii_remove = None
+            for ii, ss in enumerate(data['_extra_c_sources']):
+                if isinstance(ss, str) and '/*---GENERATED GET RECORD FUNCTION---*/' in ss:
+                   ii_remove = ii
+                   break
+            if ii_remove is not None:
+                data['_extra_c_sources'].pop(ii_remove)
+
+        data.update(data_in)
+
+        data['_isthick'] = False
+        istk = data.pop('isthick', False)
+        if istk in [True, False]:
+            data['_isthick'] = istk
+        else: # is property
+            data['isthick'] = istk
+
         # Take xofields from data['_xofields'] or from bases
         xofields = _build_xofields_dict(bases, data)
 
@@ -320,6 +439,10 @@ class MetaBeamElement(xo.MetaHybridClass):
             xofields['_shift_x'] = xo.Field(xo.Float64, 0)
             xofields['_shift_y'] = xo.Field(xo.Float64, 0)
             xofields['_shift_s'] = xo.Field(xo.Float64, 0)
+            xofields['_rot_x_rad'] = xo.Field(xo.Float64, 0)
+            xofields['_rot_y_rad'] = xo.Field(xo.Float64, 0)
+            xofields['_rot_s_rad_no_frame'] = xo.Field(xo.Float64, 0)
+            xofields['rot_shift_anchor'] = xo.Field(xo.Float64, 0)
 
         data = data.copy()
         data['_xofields'] = xofields
@@ -330,7 +453,8 @@ class MetaBeamElement(xo.MetaHybridClass):
             _pkg_root.joinpath('headers','checks.h'),
             _pkg_root.joinpath('headers','particle_states.h'),
             _pkg_root.joinpath('beam_elements', 'elements_src', 'track_srotation.h'),
-            _pkg_root.joinpath('beam_elements', 'elements_src', 'drift.h'),
+            _pkg_root.joinpath('beam_elements', 'elements_src', 'track_drift.h'),
+            c_header_flag_mapping
         ]
         kernels = {}
 
@@ -368,13 +492,16 @@ class MetaBeamElement(xo.MetaHybridClass):
 
         track_kernel_name = None
         if ('allow_track' not in data.keys() or data['allow_track']):
-
             extra_c_source.append(
                 _generate_track_local_particle_with_transformations(
                     element_name=name,
                     allow_rot_and_shift=(allow_rot_and_shift or rot_and_shift_from_parent),
                     rot_and_shift_from_parent=rot_and_shift_from_parent,
-                    local_particle_function_name=name+'_track_local_particle'))
+                    local_particle_function_name=name+'_track_local_particle',
+                    isthick=data['_isthick'],
+                    xofields=xofields,
+                )
+            )
 
             # Generate track kernel
             extra_c_source.append(
@@ -448,6 +575,9 @@ class MetaBeamElement(xo.MetaHybridClass):
             new_class.shift_x = property(_shiftx_property, _set_shiftx_property_setter)
             new_class.shift_y = property(_shifty_property, _set_shifty_property_setter)
             new_class.shift_s = property(_shifts_property, _set_shifts_property_setter)
+            new_class.rot_x_rad = property(_rot_x_property, _set_rot_x_property_setter)
+            new_class.rot_y_rad = property(_rot_y_property, _set_rot_y_property_setter)
+            new_class.rot_s_rad_no_frame = property(_rot_s_no_frame_property, _set_rot_s_no_frame_property_setter)
 
         return new_class
 
@@ -455,7 +585,6 @@ class MetaBeamElement(xo.MetaHybridClass):
 class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
 
     iscollective = False
-    isthick = False
     behaves_like_drift = False
     allow_track = True
     has_backtrack = False
@@ -468,6 +597,16 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
     def __init__(self, *args, **kwargs):
         xo.HybridClass.__init__(self, *args, **kwargs)
 
+    @property
+    def isthick(self):
+        return self._isthick
+
+    @isthick.setter
+    def isthick(self, value):
+        if value != self._isthick:
+            raise AttributeError("The property 'isthick' cannot be changed dynamically for "
+                             f"elements of type {self.__class__.__name__}")
+
     def init_pipeline(self, pipeline_manager, name, partners_names=[]):
         self._pipeline_manager = pipeline_manager
         self.name = name
@@ -480,7 +619,7 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
             partial(_handle_per_particle_blocks,
                     local_particle_src=Particles.gen_local_particle_api()))
         context = self._context
-        cls = type(self)
+        cls = self.__class__
 
         if context.allow_prebuilt_kernels:
             # Default config is empty (all flags default to not defined, which
@@ -515,6 +654,7 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
         xo.HybridClass.compile_kernels(
             self,
             extra_classes=[Particles._XoStruct],
+            extra_compile_args=(f"-I{xt.__path__[0]}",),
             *args,
             **kwargs,
         )
@@ -573,6 +713,9 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
         shift_x = kwargs.pop('shift_x', None)
         shift_y = kwargs.pop('shift_y', None)
         shift_s = kwargs.pop('shift_s', None)
+        rot_x_rad = kwargs.pop('rot_x_rad', None)
+        rot_y_rad = kwargs.pop('rot_y_rad', None)
+        rot_s_rad_no_frame = kwargs.pop('rot_s_rad_no_frame', None)
 
         xo.HybridClass.xoinitialize(self, **kwargs)
 
@@ -588,6 +731,15 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
         if shift_s is not None:
             self.shift_s = shift_s
 
+        if rot_x_rad is not None:
+            self.rot_x_rad = rot_x_rad
+
+        if rot_y_rad is not None:
+            self.rot_y_rad = rot_y_rad
+
+        if rot_s_rad_no_frame is not None:
+            self.rot_s_rad_no_frame = rot_s_rad_no_frame
+
     def to_dict(self, **kwargs):
         dct = xo.HybridClass.to_dict(self, **kwargs)
         if self.name_associated_aperture is not None:
@@ -605,6 +757,17 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
         instance.name_associated_aperture = name_associated_aperture
         return instance
 
+    def copy(self, **kwargs):
+        out = super().copy(**kwargs)
+        if hasattr(self, 'extra'):
+            try:
+                out.extra = self.extra.copy()
+            except AttributeError:
+                out.extra = self.extra
+        if hasattr(self, 'prototype'):
+            out.prototype = self.prototype
+        return out
+
     @property
     def _add_to_repr(self):
         out = []
@@ -632,6 +795,8 @@ class Replica:
         return Replica(parent_name=self.parent_name)
 
     def resolve(self, element_container, get_name=False):
+        if hasattr(element_container, 'element_dict'):
+            element_container = element_container.element_dict
         target_name = self.parent_name
         visited = {target_name}
         while isinstance(element := element_container[target_name], Replica):
