@@ -6,24 +6,27 @@
 #ifndef XTRACK_DRIFT_SLICE_H
 #define XTRACK_DRIFT_SLICE_H
 
-/*gpufun*/
+#include <headers/track.h>
+#include <beam_elements/elements_src/track_drift.h>
+
+
+GPUFUN
 void DriftSlice_track_local_particle(
         DriftSliceData el,
         LocalParticle* part0
 ) {
 
     double weight = DriftSliceData_get_weight(el);
+    double length;
+    if (LocalParticle_check_track_flag(part0, XS_FLAG_BACKTRACK)) {
+        length = -weight * DriftSliceData_get__parent_length(el); // m
+    } else {
+        length = weight * DriftSliceData_get__parent_length(el); // m
+    }
 
-    #ifndef XSUITE_BACKTRACK
-        double const length = weight * DriftSliceData_get__parent_length(el); // m
-    #else
-        double const length = -weight * DriftSliceData_get__parent_length(el); // m
-    #endif
-
-    //start_per_particle_block (part0->part)
+    START_PER_PARTICLE_BLOCK(part0, part);
         Drift_single_particle(part, length);
-    //end_per_particle_block
-
+    END_PER_PARTICLE_BLOCK;
 }
 
 #endif
