@@ -2,36 +2,27 @@
 // This file is part of the Xtrack Package.  //
 // Copyright (c) CERN, 2021.                 //
 // ######################################### //
-
 #ifndef XTRACK_XYSHIFT_H
 #define XTRACK_XYSHIFT_H
 
-
-/*gpufun*/
-void XYShift_single_particle(LocalParticle* part, double dx, double dy){
-
-    LocalParticle_add_to_x(part, -dx );
-    LocalParticle_add_to_y(part, -dy );
- 
-}
+#include <headers/track.h>
+#include <beam_elements/elements_src/track_xyshift.h>
 
 
-/*gpufun*/
+GPUFUN
 void XYShift_track_local_particle(XYShiftData el, LocalParticle* part0){
 
     double dx = XYShiftData_get_dx(el);
     double dy = XYShiftData_get_dy(el);
 
-    #ifdef XSUITE_BACKTRACK
+    if (LocalParticle_check_track_flag(part0, XS_FLAG_BACKTRACK)) {
         dx = -dx;
         dy = -dy;
-    #endif
+    }
 
-    //start_per_particle_block (part0->part)
+    START_PER_PARTICLE_BLOCK(part0, part);
         XYShift_single_particle(part, dx, dy);
-    //end_per_particle_block
-
+    END_PER_PARTICLE_BLOCK;
 }
 
-
 #endif /* XTRACK_XYSHIFT_H */
