@@ -41,6 +41,7 @@ from .progress_indicator import progress
 clight = 299792458
 
 DEFAULT_BEND_N_MULT_KICKS = 5
+DEFAULT_FIELD_ERR_NUM_KICKS = 1
 
 
 def iterable(obj):
@@ -419,8 +420,6 @@ class Alignment:
 class Dummy:
     type = "None"
 
-def _default_factory():
-    return 0.
 
 class MadLoader:
     @staticmethod
@@ -428,11 +427,11 @@ class MadLoader:
         """Enable expressions"""
         if line._var_management is None:
             line._init_var_management()
+        line.vars.default_to_zero = True
 
         from xdeps.madxutils import MadxEval
 
         _var_values = line._var_management["data"]["var_values"]
-        _var_values.default_factory = _default_factory
         for name, par in mad.globals.cmdpar.items():
             if replace_in_expr is not None:
                 for k, v in replace_in_expr.items():
@@ -496,15 +495,7 @@ class MadLoader:
             enable_align_errors = False
 
         if allow_thick is None:
-            if enable_field_errors:
-                allow_thick = False
-            else:
-                allow_thick = True
-
-        if allow_thick and enable_field_errors:
-            raise NotImplementedError(
-                "Field errors are not yet supported for thick elements"
-            )
+            allow_thick = True
 
         if expressions_for_element_types is not None:
             assert enable_expressions, ("Expressions must be enabled if "
@@ -662,8 +653,7 @@ class MadLoader:
         if issubclass(self.Builder, ElementBuilderWithExpr):
             return self.line._var_management['fref']
 
-        import math
-        return math
+        return np
 
     def _assert_element_is_thin(self, mad_el):
         if value_if_expr(mad_el.l) != 0:
@@ -752,7 +742,7 @@ class MadLoader:
         if self.allow_thick:
             if not mad_el.l:
                 raise ValueError(
-                    "Thick quadrupole with legth zero are not supported.")
+                    "Thick quadrupole with length zero are not supported.")
             return self._convert_quadrupole_thick(mad_el)
         else:
             raise NotImplementedError(
@@ -760,6 +750,10 @@ class MadLoader:
             )
 
     def _convert_quadrupole_thick(self, mad_el): # bv done
+        kwargs = {}
+        if self.enable_field_errors:
+            kwargs = _prepare_field_errors_thick_elem(mad_el)
+            kwargs['num_multipole_kicks'] = 1
 
         return self.make_composite_element(
             [
@@ -769,6 +763,7 @@ class MadLoader:
                     k1=self.bv * mad_el.k1,
                     k1s=mad_el.k1s,
                     length=mad_el.l,
+                    **kwargs,
                 ),
             ],
             mad_el,
@@ -784,60 +779,76 @@ class MadLoader:
         self,
         mad_el,
     ):
-
         assert self.allow_thick, "Bends are not supported in thin mode."
 
-        l_curv = mad_el.l
-        h = mad_el.angle / l_curv
-
-        if mad_el.type == 'rbend' and self.sequence._madx.options.rbarc and value_if_expr(mad_el.angle):
-            R = 0.5 * mad_el.l / self.math.sin(0.5 * mad_el.angle) # l is on the straight line
-            l_curv = R * mad_el.angle
-            h = 1 / R
-
-        if not mad_el.k0:
-            k0 = h
+        if mad_el.type == 'sbend':
+            element_type = self.classes.Bend
+        elif mad_el.type == 'rbend':
+            element_type = self.classes.RBend
         else:
-            k0 = mad_el.k0
+            raise ValueError(f'Unknown bend type {mad_el.type}.')
+
+        bend_kwargs = {}
+
+        if mad_el.type == 'rbend' and self.sequence._madx.options.rbarc:
+            l_curv = mad_el.l / self.math.sinc(0.5 * mad_el.angle)
+            bend_kwargs['length_straight'] = mad_el.l
+        else:
+            l_curv = mad_el.l
+            bend_kwargs['length'] = l_curv
+
+        bend_kwargs['angle'] = mad_el.angle
 
         # Edge angles
-        if mad_el.type == 'sbend':
-            e1 = mad_el.e1
-            e2 = mad_el.e2
-        elif mad_el.type == 'rbend':
-            e1 = mad_el.e1 + mad_el.angle / 2
-            e2 = mad_el.e2 + mad_el.angle / 2
-        else:
-            raise NotImplementedError(
-                f'Unknown bend type {mad_el.type}.'
-            )
-
+        e1 = mad_el.e1
+        e2 = mad_el.e2
         if self.bv == -1:
             e1, e2 = e2, e1
 
+        # Edge angles for field errors
+        if mad_el.k0:
+            h = mad_el.angle / l_curv
+            angle_fdown = (mad_el.k0 - h) * l_curv / 2
+        else:
+            angle_fdown = 0
+
+        if self.enable_field_errors:
+            kwargs = _prepare_field_errors_thick_elem(mad_el)
+            knl = kwargs['knl']
+            ksl = kwargs['ksl']
+            num_multipole_kicks = 1
+        else:
+            knl = [0] * 3
+            ksl = []
+            num_multipole_kicks = 0
+
+        knl[2] += mad_el.k2 * l_curv
+
+        if mad_el.k0:
+            k0_from_h = False
+            bend_kwargs['k0'] = mad_el.k0
+        else:
+            k0_from_h = True
+
         # Convert bend core
-        num_multipole_kicks = 0
-        cls = self.classes.Bend
-        kwargs = {}
         bend_core = self.Builder(
             mad_el.name,
-            cls,
-            k0=k0,
-            h=h,
+            element_type,
+            k0_from_h=k0_from_h,
             k1=self.bv * mad_el.k1,
-            length=l_curv,
             edge_entry_angle=e1,
             edge_exit_angle=e2,
-            edge_entry_angle_fdown=(k0 - h) * l_curv / 2,
-            edge_exit_angle_fdown=(k0 - h) * l_curv / 2,
+            edge_entry_angle_fdown=angle_fdown,
+            edge_exit_angle_fdown=angle_fdown,
             edge_entry_fint=mad_el.fint,
             edge_exit_fint=(
                 mad_el.fintx if value_if_expr(mad_el.fintx) >= 0 else mad_el.fint),
             edge_entry_hgap=mad_el.hgap,
             edge_exit_hgap=mad_el.hgap,
-            knl=[0, 0, mad_el.k2 * l_curv],
+            knl=knl,
+            ksl=ksl,
             num_multipole_kicks=num_multipole_kicks,
-            **kwargs,
+            **bend_kwargs,
         )
 
         sequence = [bend_core]
@@ -845,6 +856,11 @@ class MadLoader:
         return self.make_composite_element(sequence, mad_el)
 
     def convert_sextupole(self, mad_el): # bv done
+        kwargs = {}
+
+        if self.enable_field_errors:
+            kwargs = _prepare_field_errors_thick_elem(mad_el)
+
         return self.make_composite_element(
             [
                 self.Builder(
@@ -853,12 +869,18 @@ class MadLoader:
                     k2=mad_el.k2,
                     k2s=self.bv * mad_el.k2s,
                     length=mad_el.l,
+                    **kwargs,
                 ),
             ],
             mad_el,
         )
 
     def convert_octupole(self, mad_el): # bv done
+        kwargs = {}
+
+        if self.enable_field_errors:
+            kwargs = _prepare_field_errors_thick_elem(mad_el)
+
         return self.make_composite_element(
             [
                 self.Builder(
@@ -867,6 +889,7 @@ class MadLoader:
                     k3=self.bv*mad_el.k3,
                     k3s=mad_el.k3s,
                     length=mad_el.l,
+                    **kwargs,
                 ),
             ],
             mad_el,
@@ -978,12 +1001,17 @@ class MadLoader:
                    f'reverting to importing `{mad_elem.name}` as a drift.')
             return self.convert_drift_like(mad_elem)
 
+        kwargs = {}
+
+        if self.enable_field_errors:
+            kwargs = _prepare_field_errors_thick_elem(mad_elem)
+
         el = self.Builder(
             mad_elem.name,
-            self.classes.Solenoid,
+            self.classes.UniformSolenoid,
             length=mad_elem.l,
             ks=self.bv * mad_elem.ks,
-            ksi=self.bv * mad_elem.ksi,
+            **kwargs,
         )
         return self.make_composite_element([el], mad_elem)
 
@@ -1003,51 +1031,57 @@ class MadLoader:
             lmax = max(lmax, non_zero_len(dkn), non_zero_len(dks))
             knl = add_lists(knl, dkn, lmax)
             ksl = add_lists(ksl, dks, lmax)
-        el = self.Builder(mad_elem.name, self.classes.Multipole, order=lmax - 1)
-        el.knl = knl[:lmax]
-        el.ksl = ksl[:lmax]
 
         if hasattr(mad_elem, 'ksl') and mad_elem.ksl[0]:
             raise NotImplementedError("Multipole with ksl[0] is not supported.")
 
-        if hasattr(el, 'hyl') and el.hyl:
+        if hasattr(mad_elem, 'hyl') and mad_elem.hyl:
             raise NotImplementedError("Multipole with hyl is not supported.")
 
-        if (
-            mad_elem.angle
-        ):  # testing for non-zero (cannot use !=0 as it creates an expression)
+        el = self.Builder(mad_elem.name, self.classes.Multipole, order=lmax - 1)
+        el.knl = knl[:lmax]
+        el.ksl = ksl[:lmax]
+
+        if mad_elem.angle:  # testing for non-zero (!=0 would create an expression)
             el.hxl = mad_elem.angle
         else:
             el.hxl = mad_elem.knl[0]  # in madx angle=0 -> dipole
         el.length = mad_elem.lrad
         return self.make_composite_element([el], mad_elem)
 
-    def convert_kicker(self, mad_el): # bv done
-        hkick = [-mad_el.hkick] if mad_el.hkick else []
-        vkick = [self.bv * mad_el.vkick] if mad_el.vkick else []
-        thin_kicker = self.Builder(
+    def _make_kicker_multipole(self, mad_el, hkick, vkick):
+
+        if mad_el.l:
+            isthick = True
+            length = mad_el.l
+        else:
+            isthick = False
+            length = mad_el.lrad
+
+
+        kicker = self.Builder(
             mad_el.name,
             self.classes.Multipole,
             knl=hkick,
             ksl=vkick,
-            length=(mad_el.l or mad_el.lrad),
-            hxl=0,
+            length=length,
+            isthick=isthick,
         )
 
-        if value_if_expr(mad_el.l) != 0:
-            if not self.allow_thick:
-                self._assert_element_is_thin(mad_el)
+        if isthick and not self.allow_thick:
+            self._assert_element_is_thin(mad_el)
 
-            sequence = [
-                self._make_drift_slice(mad_el, 0.5, "drift_{}..1"),
-                thin_kicker,
-                self._make_drift_slice(mad_el, 0.5, "drift_{}..2"),
-            ]
-        else:
-            sequence = [thin_kicker]
+        sequence = [kicker]
 
         return self.make_composite_element(sequence, mad_el)
 
+
+    def convert_kicker(self, mad_el): # bv done
+        hkick = [-mad_el.hkick] if mad_el.hkick else []
+        vkick = [self.bv * mad_el.vkick] if mad_el.vkick else []
+
+        return self._make_kicker_multipole(mad_el, hkick, vkick)
+
     convert_tkicker = convert_kicker
 
     def convert_hkicker(self, mad_el): # bv done
@@ -1057,28 +1091,8 @@ class MadLoader:
 
         hkick = [-mad_el.kick] if mad_el.kick else []
         vkick = []
-        thin_hkicker = self.Builder(
-            mad_el.name,
-            self.classes.Multipole,
-            knl=hkick,
-            ksl=vkick,
-            length=(mad_el.l or mad_el.lrad),
-            hxl=0,
-        )
 
-        if value_if_expr(mad_el.l) != 0:
-            if not self.allow_thick:
-                self._assert_element_is_thin(mad_el)
-
-            sequence = [
-                self._make_drift_slice(mad_el, 0.5, "drift_{}..1"),
-                thin_hkicker,
-                self._make_drift_slice(mad_el, 0.5, "drift_{}..2"),
-            ]
-        else:
-            sequence = [thin_hkicker]
-
-        return self.make_composite_element(sequence, mad_el)
+        return self._make_kicker_multipole(mad_el, hkick, vkick)
 
     def convert_vkicker(self, mad_el): # bv done
         if mad_el.vkick:
@@ -1087,28 +1101,8 @@ class MadLoader:
 
         hkick = []
         vkick = [self.bv * mad_el.kick] if mad_el.kick else []
-        thin_vkicker = self.Builder(
-            mad_el.name,
-            self.classes.Multipole,
-            knl=hkick,
-            ksl=vkick,
-            length=(mad_el.l or mad_el.lrad),
-            hxl=0,
-        )
 
-        if value_if_expr(mad_el.l) != 0:
-            if not self.allow_thick:
-                self._assert_element_is_thin(mad_el)
-
-            sequence = [
-                self._make_drift_slice(mad_el, 0.5, "drift_{}..1"),
-                thin_vkicker,
-                self._make_drift_slice(mad_el, 0.5, "drift_{}..2"),
-            ]
-        else:
-            sequence = [thin_vkicker]
-
-        return self.make_composite_element(sequence, mad_el)
+        return self._make_kicker_multipole(mad_el, hkick, vkick)
 
     def convert_dipedge(self, mad_elem):
         if self.bv == -1:
@@ -1149,40 +1143,38 @@ class MadLoader:
             voltage=scale_voltage * ee.volt * 1e6,
             frequency=frequency,
             lag=lag_deg,
+            length=ee.l
         )
 
-        if value_if_expr(ee.l) != 0:
-            sequence = [
-                self._make_drift_slice(ee, 0.5, f"drift_{{}}..1"),
-                el,
-                self._make_drift_slice(ee, 0.5, f"drift_{{}}..2"),
-            ]
-        else:
-            sequence = [el]
+        sequence = [el]
 
         return self.make_composite_element(sequence, ee)
 
     def convert_rfmultipole(self, ee):
-        if self.bv == -1:
-            raise NotImplementedError("RF multipole for bv=-1 are not yet supported.")
-        self._assert_element_is_thin(ee)
-        # TODO LRAD
-        if ee.harmon:
-            raise NotImplementedError
-        if ee.l:
-            raise NotImplementedError
-        el = self.Builder(
-            ee.name,
-            self.classes.RFMultipole,
-            voltage=ee.volt * 1e6,
-            frequency=ee.freq * 1e6,
-            lag=ee.lag * 360,
-            knl=ee.knl,
-            ksl=ee.ksl,
-            pn=[v * 360 for v in ee.pnl],
-            ps=[v * 360 for v in ee.psl],
-        )
-        return self.make_composite_element([el], ee)
+        raise NotImplementedError('Conversion of mad-x rfmultipole not supported')
+
+        # The following is untested, espeically for bv=-1
+
+        # if self.bv == -1:
+        #     raise NotImplementedError("RF multipole for bv=-1 are not yet supported.")
+        # self._assert_element_is_thin(ee)
+        # # TODO LRAD
+        # if ee.harmon:
+        #     raise NotImplementedError
+        # if ee.l:
+        #     raise NotImplementedError
+        # el = self.Builder(
+        #     ee.name,
+        #     self.classes.RFMultipole,
+        #     voltage=ee.volt * 1e6,
+        #     frequency=ee.freq * 1e6,
+        #     lag=ee.lag * 360,
+        #     knl=ee.knl,
+        #     ksl=ee.ksl,
+        #     pn=[v * 360 for v in ee.pnl],
+        #     ps=[v * 360 for v in ee.psl],
+        # )
+        # return self.make_composite_element([el], ee)
 
     def convert_wire(self, ee):
         if self.bv == -1:
@@ -1207,32 +1199,17 @@ class MadLoader:
 
     def convert_crabcavity(self, ee):
         if self.bv == -1:
-            raise NotImplementedError("Crab cavity for bv=-1 are not yet supported.")
-        self._assert_element_is_thin(ee)
-        # This has to be disabled, as it raises an error when l is assigned to an
-        # expression:
-        # for nn in ["l", "harmon", "lagf", "rv1", "rv2", "rph1", "rph2"]:
-        #     if getattr(ee, nn):
-        #         raise NotImplementedError(f"Invalid value {nn}={getattr(ee, nn)}")
-
-        # ee.volt in MV, sequence.beam.pc in GeV
-        if abs(ee.tilt - np.pi / 2) < 1e-9:
-            el = self.Builder(
-                ee.name,
-                self.classes.RFMultipole,
-                frequency=ee.freq * 1e6,
-                ksl=[-ee.volt / self.sequence.beam.pc * 1e-3],
-                ps=[ee.lag * 360 + 90],
-            )
-            ee.tilt = 0
+            lll = 180 - ee.lag * 360
         else:
-            el = self.Builder(
+            lll = ee.lag * 360
+
+        el = self.Builder(
                 ee.name,
-                self.classes.RFMultipole,
+                self.classes.CrabCavity,
+                length=ee.l,
                 frequency=ee.freq * 1e6,
-                knl=[ee.volt / self.sequence.beam.pc * 1e-3],
-                pn=[ee.lag * 360 + 90],  # TODO: Changed sign to match sixtrack
-                # To be checked!!!!
+                crab_voltage=ee.volt * 1e6 * self.bv,
+                lag=lll,
             )
         return self.make_composite_element([el], ee)
 
@@ -1395,3 +1372,28 @@ class MadLoader:
             cnll=mad_elem.cnll,
         )
         return self.make_composite_element([el], mad_elem)
+
+
+def _prepare_field_errors_thick_elem(mad_el):
+    if mad_el.field_errors is None:
+        return {}
+
+    dkn = mad_el.field_errors.dkn
+    dks = mad_el.field_errors.dks
+    lmax = max(non_zero_len(dkn), non_zero_len(dks))
+    if lmax > 6:
+        raise ValueError(
+            "Only up to dodecapoles are supported for field errors"
+            " of thick magnets for now."
+        )
+    if len(dkn) > lmax:
+        dkn = dkn[:lmax]
+    if len(dks) > lmax:
+        dks = dks[:lmax]
+
+    kwargs_to_add = {}
+    if np.any(np.abs(dkn)) or np.any(np.abs(dks)):
+        kwargs_to_add['knl'] = dkn
+        kwargs_to_add['ksl'] = dks
+
+    return kwargs_to_add
