@@ -2,6 +2,7 @@
 # This file is part of the Xtrack Package.  #
 # Copyright (c) CERN, 2023.                 #
 # ######################################### #
+import json
 
 import numpy as np
 from pathlib import Path
@@ -11,11 +12,14 @@ from scipy.constants import c as clight
 from scipy.constants import epsilon_0
 
 import xobjects as xo
+import xtrack as xt
 from xobjects.general import Print
 from xobjects import BypassLinked
 
-from .constants import PROTON_MASS_EV
-
+from .masses import PROTON_MASS_EV
+from .masses import __dict__ as mass__dict__
+from .pdg import get_pdg_id_from_name, get_properties_from_pdg_id, \
+                 get_mass_from_pdg_id
 
 LAST_INVALID_STATE = -999999999
 
@@ -65,6 +69,10 @@ class Particles(xo.HybridClass):
             (xo.Float64, 'weight'),
             (xo.Float64, 'ax'),
             (xo.Float64, 'ay'),
+            (xo.Float64, 'spin_x'),
+            (xo.Float64, 'spin_y'),
+            (xo.Float64, 'spin_z'),
+            (xo.Float64, 'anomalous_magnetic_moment'),
             (xo.Int64, 'pdg_id'),
             (xo.Int64, 'particle_id'),
             (xo.Int64, 'at_element'),
@@ -211,6 +219,14 @@ class Particles(xo.HybridClass):
             raise NameError('`psigma` is not supported anymore.'
                             'Please use `pzeta` instead.')
 
+        accepted_args = set(self._xofields.keys()) | {
+            'energy0', 'tau', 'pzeta', 'mass_ratio', 'mass', 'kinetic_energy0',
+            '_context', '_buffer', '_offset', 'p0', 'name',
+        }
+        if set(kwargs.keys()) - accepted_args:
+            raise NameError(f'Invalid argument(s) provided: '
+                            f'{set(kwargs.keys()) - accepted_args}')
+
         per_part_input_vars = (
             self.per_particle_vars +
             ((xo.Float64, 'energy0'),
@@ -338,6 +354,14 @@ class Particles(xo.HybridClass):
             mask=input_mask,
         )
 
+        # Init chi and charge ratio
+        self._update_chi_charge_ratio(
+            chi=kwargs.get('chi'),
+            charge_ratio=kwargs.get('charge_ratio'),
+            mass_ratio=kwargs.get('mass_ratio'),
+            mask=input_mask,
+        )
+
         # Init energy deviations
         self._update_energy_deviations(
             delta=kwargs.get('delta'),
@@ -355,18 +379,12 @@ class Particles(xo.HybridClass):
             mask=input_mask,
         )
 
-        # Init chi and charge ratio
-        self._update_chi_charge_ratio(
-            chi=kwargs.get('chi'),
-            charge_ratio=kwargs.get('charge_ratio'),
-            mass_ratio=kwargs.get('mass_ratio'),
-            mask=input_mask,
-        )
-
         self.unhide_first_n_particles()
         if isinstance(self._context, xo.ContextCpu) and not _no_reorganize:
             self.reorganize()
 
+        if 'name' in kwargs.keys():
+            self.name = kwargs['name']
 
     @classmethod
     def from_dict(cls, dct, load_rng_state=True, **kwargs):
@@ -490,8 +508,35 @@ class Particles(xo.HybridClass):
 
         return dct
 
+    def to_json(self, filename, indent=None, **kwargs):
+        """
+        Save the Particles object to a JSON file.
+
+        Parameters
+        ----------
+        filename : str
+            The name of the file to save the Particles object to.
+        **kwargs : dict
+            Additional keyword arguments to pass to the json.to_dict method.
+        """
+
+        class NumpyEncoder(json.JSONEncoder):
+            def default(self, obj):
+                if isinstance(obj, np.ndarray):
+                    return obj.tolist()
+                if isinstance(obj, np.floating):
+                    return float(obj)
+                if isinstance(obj, np.integer):
+                    return int(obj)
+                return json.JSONEncoder.default(self, obj)
+
+        dct = self.to_dict(**kwargs)
+
+        with open(filename, 'w') as f:
+            json.dump(dct, f, cls=NumpyEncoder, indent=indent)
+
     @classmethod
-    def from_pandas(cls, df, _context=None, _buffer=None, _offset=None):
+    def from_pandas(cls, df, _context=None, _buffer=None, _offset=None, load_rng_state=True, **kwargs):
 
         """
         Create a new Particles object from a pandas DataFrame.
@@ -517,7 +562,9 @@ class Particles(xo.HybridClass):
         for tt, nn in cls.scalar_vars + cls.size_vars:
             if nn in dct.keys() and not np.isscalar(dct[nn]):
                 dct[nn] = dct[nn][0]
-        return cls(**dct, _context=_context, _buffer=_buffer, _offset=_offset)
+        return cls.from_dict(dct, load_rng_state=load_rng_state,
+                             _context=_context, _buffer=_buffer,
+                             _offset=_offset, **kwargs)
 
     def to_pandas(self,
                   remove_underscored=None,
@@ -732,6 +779,8 @@ class Particles(xo.HybridClass):
         capacity = len(test_x)
         new_part_cpu = self.__class__(_capacity=capacity)
 
+        new_part_cpu.start_tracking_at_element = self.start_tracking_at_element
+
         # Copy scalar vars from first particle
         for tt, nn in self.scalar_vars:
             setattr(new_part_cpu, nn, getattr(self_cpu, nn))
@@ -747,7 +796,7 @@ class Particles(xo.HybridClass):
         # Copy to original context
         target_ctx = self._buffer.context
         if isinstance(target_ctx, xo.ContextCpu):
-            new_part_cpu._buffer.context = target_ctx
+            new_part_cpu.move(_context=target_ctx)
             return new_part_cpu
         else:
             return new_part_cpu.copy(_context=target_ctx)
@@ -846,16 +895,15 @@ class Particles(xo.HybridClass):
             # Reorganize particles
             with self._bypass_linked_vars():
                 for tt, nn in self.per_particle_vars:
+                    if nn.startswith('_rng'):
+                        continue
                     vv = getattr(self, nn)
                     vv_active = vv[mask_active]
                     vv_lost = vv[mask_lost]
 
                     vv[:n_active] = vv_active
                     vv[n_active:n_active + n_lost] = vv_lost
-                    if nn.startswith('_rng'):
-                        vv[n_active + n_lost:] = 0
-                    else:
-                        vv[n_active + n_lost:] = tt._dtype.type(LAST_INVALID_STATE)
+                    vv[n_active + n_lost:] = tt._dtype.type(LAST_INVALID_STATE)
 
         if isinstance(self._buffer.context, xo.ContextCpu):
             self._num_active_particles = n_active
@@ -958,6 +1006,7 @@ class Particles(xo.HybridClass):
     def init_pipeline(self, name):
 
         self.name = name
+        self._needs_pipeline = True
 
     def show(self):
 
@@ -1025,7 +1074,10 @@ class Particles(xo.HybridClass):
                     kernel_descriptions=self._kernels,
                 )
                 context.kernels.update(kernels)
-        self.compile_kernels(only_if_needed=True)
+        self.compile_kernels(
+            only_if_needed=True,
+            extra_compile_args=(f"-I{xt.__path__[0]}",),
+        )
 
         if seeds is None:
             seeds = np.random.randint(low=1, high=4e9,
@@ -1352,7 +1404,7 @@ class Particles(xo.HybridClass):
         Add `delta_energy` to the `energy` of the particles object. `delta`,
         'ptau', `rvv` and `rpp` are updated accordingly.
         """
-        self.ptau += delta_energy / self.p0c * self.mass_ratio
+        self.ptau += delta_energy / self.p0c / self.mass_ratio
 
     def set_particle(self, index, set_scalar_vars=False, **kwargs):
         raise NotImplementedError('This functionality has been removed')
@@ -1362,7 +1414,11 @@ class Particles(xo.HybridClass):
         if mode != 'no_local_copy':
             raise NotImplementedError
 
-        src_lines = []
+        src_lines = ['']
+        for name, mass in mass__dict__.items():
+            if name.endswith('_MASS_EV'):
+                src_lines.append(f'#define {name} {mass}')
+
         src_lines.append('typedef struct {')
 
         for tt, vv in cls.size_vars + cls.scalar_vars:
@@ -1373,6 +1429,7 @@ class Particles(xo.HybridClass):
 
         src_lines.append('                 int64_t ipart;')
         src_lines.append('                 int64_t endpart;')
+        src_lines.append('                 uint64_t track_flags;')
         src_lines.append('    /*gpuglmem*/ int8_t* io_buffer;')
         src_lines.append('} LocalParticle;')
         src_typedef = '\n'.join(src_lines)
@@ -1387,6 +1444,14 @@ class Particles(xo.HybridClass):
 
             ''')
 
+        # Get track flag
+        src_lines.append('''
+            /*gpufun*/
+            uint64_t LocalParticle_check_track_flag(LocalParticle* part, uint8_t index){
+                return (part->track_flags >> index) & 1;
+            }
+        ''')
+
         # Particles_to_LocalParticle
         src_lines.append('''
             /*gpufun*/
@@ -1411,8 +1476,7 @@ class Particles(xo.HybridClass):
         src_lines = []
         src_lines.append('''
             /*gpufun*/
-            void LocalParticle_to_Particles(
-                                            LocalParticle* source,
+            void LocalParticle_to_Particles(LocalParticle* source,
                                             ParticlesData dest,
                                             int64_t id,
                                             int64_t set_scalar){''')
@@ -1703,6 +1767,9 @@ class Particles(xo.HybridClass):
         int64_t check_is_active(LocalParticle* part) {
             int64_t ipart=0;
             while (ipart < part->_num_active_particles){
+                #ifdef XSUITE_RESTORE_LOSS
+                ipart++;
+                #else
                 if (part->state[ipart]<1){
                     LocalParticle_exchange(
                         part, ipart, part->_num_active_particles-1);
@@ -1712,6 +1779,7 @@ class Particles(xo.HybridClass):
                 else{
                     ipart++;
                 }
+                #endif
             }
 
             if (part->_num_active_particles==0){
@@ -1823,9 +1891,9 @@ class Particles(xo.HybridClass):
                         double const p0c = LocalParticle_get_p0c(part);
                         double const charge_ratio = LocalParticle_get_charge_ratio(part);
                         double const chi = LocalParticle_get_chi(part);
-                        double const mass_ratio = chi / charge_ratio;
-                        
-                        ptau += delta_energy/p0c * mass_ratio;
+                        double const mass_ratio = charge_ratio / chi;
+
+                        ptau += delta_energy/p0c / mass_ratio;
 
                         double const old_rpp = LocalParticle_get_rpp(part);
 
@@ -1896,10 +1964,13 @@ class Particles(xo.HybridClass):
         self.py = kwargs.get('py', 0)
         self.ax = kwargs.get('ax', 0)
         self.ay = kwargs.get('ay', 0)
+        self.anomalous_magnetic_moment = kwargs.get('anomalous_magnetic_moment', 0)
+        self.spin_x = kwargs.get('spin_x', 0)
+        self.spin_y = kwargs.get('spin_y', 0)
+        self.spin_z = kwargs.get('spin_z', 0)
 
         pdg_id = kwargs.get('pdg_id')
         try:
-            from xpart.pdg import get_pdg_id_from_name
             pdg_id = get_pdg_id_from_name(pdg_id)
             if not np.isscalar(pdg_id):
                 pdg_id = self._context.nparray_to_context_array(pdg_id)
@@ -1913,9 +1984,6 @@ class Particles(xo.HybridClass):
 
     @classmethod
     def reference_from_pdg_id(cls, pdg_id, **kwargs):
-        from xpart.pdg import (
-            get_pdg_id_from_name, get_properties_from_pdg_id, get_mass_from_pdg_id
-        )
         pdg_id = get_pdg_id_from_name(pdg_id)
         kwargs['pdg_id'] = pdg_id
         q0 = kwargs.get('q0')
@@ -2037,6 +2105,9 @@ class Particles(xo.HybridClass):
             if _rpp is not None or _rvv is not None:
                 raise ValueError('Setting `delta` and `ptau` by only giving '
                                  '`_rpp` and `_rvv` is not supported.')
+            if any(self.mass_ratio != 1.0):
+                raise ValueError('Need to provide `delta` or `ptau` with '
+                                 'non-default mass ratios.')
             self._delta = 0.0
             delta = self._delta  # Cupy complains if we later assign LinkedArray
 
@@ -2143,7 +2214,7 @@ class Particles(xo.HybridClass):
                                     computed_value=_charge_ratio,
                                     mask=mask)
 
-    def update_p0c_and_energy_deviations(self, p0c):
+    def update_p0c_and_energy_deviations(self, p0c, update_pxpy=False):
 
         assert np.isscalar(p0c), 'p0c must be a scalar'
 
@@ -2163,6 +2234,13 @@ class Particles(xo.HybridClass):
         self._update_energy_deviations(mask=mask, delta=new_delta)
         self._update_zeta(mask=mask, zeta=self.zeta * self.beta0 / old_beta0)
 
+        if update_pxpy:
+            if isinstance(self._context, xo.ContextPyopencl):
+                raise NotImplementedError # Issue wiht masking
+            scale_pxpy = old_p0c[mask] / new_p0c[mask]
+            self.px[mask] *= scale_pxpy
+            self.py[mask] *= scale_pxpy
+
 
 def _mask_to_where(mask, ctx):
     if hasattr(mask, 'get'):
