@@ -23,13 +23,10 @@ from .line import freeze_longitudinal as _freeze_longitudinal
 from .pipeline import PipelineStatus
 from .progress_indicator import progress
 from .tracker_data import TrackerData
+from .track_flags import TrackFlags
 
 logger = logging.getLogger(__name__)
 
-
-
-
-
 class Tracker:
 
     '''
@@ -65,10 +62,6 @@ class Tracker:
             raise ValueError("`enable_pipeline_hold` is not implemented in "
                              "non-collective mode")
 
-        if not compile and self.iscollective:
-            raise NotImplementedError("Skip compilation is not implemented in "
-                                      "collective mode")
-
         if local_particle_src is None:
             local_particle_src = xt.Particles.gen_local_particle_api()
 
@@ -81,6 +74,7 @@ class Tracker:
         self.local_particle_src = local_particle_src
         self._enable_pipeline_hold = enable_pipeline_hold
         self.use_prebuilt_kernels = use_prebuilt_kernels
+        self.track_flags = TrackFlags()
 
         # Some data for collective mode prepared also for non-collective lines
         # to allow collective actions by the tracker (e.g. time-functions on knobs)
@@ -124,6 +118,10 @@ class Tracker:
             _context=_context,
             _buffer=_buffer,
             _no_resolve_parents=_prebuilding_kernels)
+        if not _prebuilding_kernels:
+            tracker_data_base._line_table = line.get_table()
+            tracker_data_base._element_names_unique = tuple(
+                    tracker_data_base._line_table.name[:-1]) # remove _endpoint
         line._freeze()
 
         if np.any([hasattr(ee, 'needs_rng') and ee.needs_rng for ee in line.elements]):
@@ -151,7 +149,8 @@ class Tracker:
 
     def _init_io_buffer(self, io_buffer=None):
         if io_buffer is None:
-            io_bufs = [ee.io_buffer for ee in self.line.elements if hasattr(ee, 'io_buffer')]
+            io_bufs = [ee.io_buffer for ee in self.line.elements
+                       if getattr(ee, 'io_buffer', None) is not None]
             if len(io_bufs) == 0:
                 io_buffer = new_io_buffer(_context=self._context)
             elif len(np.unique([id(buf) for buf in io_bufs])) > 1:
@@ -304,7 +303,8 @@ class Tracker:
             t0 = perf_counter()
 
         assert self.iscollective in (True, False)
-        if self.iscollective or self.line.enable_time_dependent_vars:
+        if (self.iscollective or self.line.enable_time_dependent_vars
+            or 'log' in kwargs and kwargs['log'] is not None):
             tracking_func = self._track_with_collective
         else:
             tracking_func = self._track_no_collective
@@ -466,6 +466,7 @@ class Tracker:
 
         headers.extend(self.extra_headers)
         headers.append(_pkg_root.joinpath("headers/constants.h"))
+        headers.append(self.track_flags.c_header_flag_mapping)
 
         src_lines = []
         src_lines.append(
@@ -485,7 +486,9 @@ class Tracker:
                              double line_length,
                 /*gpuglmem*/ int8_t* buffer_tbt_monitor,
                              int64_t offset_tbt_monitor,
-                /*gpuglmem*/ int8_t* io_buffer){
+                /*gpuglmem*/ int8_t* io_buffer,
+                             uint64_t track_flags
+                             ){
 
             #define CONTEXT_OPENMP  //only_for_context cpu_openmp
             #ifdef CONTEXT_OPENMP
@@ -494,7 +497,7 @@ class Tracker:
 
                 #ifndef XT_OMP_SKIP_REORGANIZE
                     const int64_t num_particles_to_track = ParticlesData_get__num_active_particles(particles);
-                    
+
                     {
                         LocalParticle lpart;
                         lpart.io_buffer = io_buffer;
@@ -506,10 +509,10 @@ class Tracker:
                 #else // When we skip reorganize, we cannot just batch active particles
                     const int64_t num_particles_to_track = capacity;
                 #endif
-                
+
                 const int64_t chunk_size = (num_particles_to_track + num_threads - 1)/num_threads; // ceil division
             #endif // CONTEXT_OPENMP
-            
+
             #pragma omp parallel for                                                           //only_for_context cpu_openmp
             for (int chunk = 0; chunk < num_threads; chunk++) {                                //only_for_context cpu_openmp
             int64_t part_id = chunk * chunk_size;                                              //only_for_context cpu_openmp
@@ -523,6 +526,7 @@ class Tracker:
 
             LocalParticle lpart;
             lpart.io_buffer = io_buffer;
+            lpart.track_flags = track_flags;
 
             /*gpuglmem*/ int8_t* tbt_mon_pointer =
                             buffer_tbt_monitor + offset_tbt_monitor;
@@ -543,20 +547,22 @@ class Tracker:
 
                 int64_t const ele_stop = ele_start + num_ele_track;
 
-                #ifndef XSUITE_BACKTRACK
-                if (flag_monitor==1){
-                    ParticlesMonitor_track_local_particle(tbt_monitor, &lpart);
+                int64_t elem_idx, increm;
+                if (LocalParticle_check_track_flag(&lpart, XS_FLAG_BACKTRACK)) {
+                    elem_idx = ele_stop - 1;
+                    increm = -1;
+                    if (flag_end_turn_actions>0){
+                        increment_at_turn_backtrack(&lpart, flag_reset_s_at_end_turn,
+                                                    line_length, num_ele_line);
+                    }
                 }
-                int64_t elem_idx = ele_start;
-                int64_t const increm = 1;
-                #else
-                int64_t elem_idx = ele_stop - 1;
-                int64_t const increm = -1;
-                if (flag_end_turn_actions>0){
-                    increment_at_turn_backtrack(&lpart, flag_reset_s_at_end_turn,
-                                                line_length, num_ele_line);
+                else{
+                    if (flag_monitor==1){
+                        ParticlesMonitor_track_local_particle(tbt_monitor, &lpart);
+                    }
+                    elem_idx = ele_start;
+                    increm = 1;
                 }
-                #endif
 
                 for (; ((elem_idx >= ele_start) && (elem_idx < ele_stop)); elem_idx+=increm){
                         if (flag_monitor==2){
@@ -606,11 +612,11 @@ class Tracker:
                         break;
                     }
 
-                    #ifndef XSUITE_BACKTRACK
-                        increment_at_element(&lpart, 1);
-                    #else
+                    if (LocalParticle_check_track_flag(&lpart, XS_FLAG_BACKTRACK)) {
                         increment_at_element(&lpart, -1);
-                    #endif //XSUITE_BACKTRACK
+                    } else {
+                        increment_at_element(&lpart, 1);
+                    }
 
                     #endif //DANGER_SKIP_ACTIVE_CHECK_AND_SWAPS
 
@@ -621,20 +627,16 @@ class Tracker:
                     ParticlesMonitor_track_local_particle(tbt_monitor, &lpart);
                 }
 
-                #ifndef XSUITE_BACKTRACK
-                if (flag_end_turn_actions>0){
+                if (LocalParticle_check_track_flag(&lpart, XS_FLAG_BACKTRACK)) {
+                    if (flag_monitor==1){
+                        ParticlesMonitor_track_local_particle(tbt_monitor, &lpart);
+                    }
+                }
+                else if (flag_end_turn_actions>0){
                     if (isactive){
                         increment_at_turn(&lpart, flag_reset_s_at_end_turn);
                     }
                 }
-                #endif //XSUITE_BACKTRACK
-
-
-                #ifdef XSUITE_BACKTRACK
-                if (flag_monitor==1){
-                    ParticlesMonitor_track_local_particle(tbt_monitor, &lpart);
-                }
-                #endif //XSUITE_BACKTRACK
             } // for turns
 
             LocalParticle_to_Particles(&lpart, particles, part_id, 1);
@@ -682,6 +684,7 @@ class Tracker:
             specialize=True,
             compile=compile,
             save_source_as=f'{module_name}.c' if module_name else None,
+            extra_compile_args=(f"-I{xt.__path__[0]}",),
             **kwargs,
         )
         return out_kernels['track_line']
@@ -709,6 +712,7 @@ class Tracker:
                     xo.Arg(xo.Int8, pointer=True, name="buffer_tbt_monitor"),
                     xo.Arg(xo.Int64, name="offset_tbt_monitor"),
                     xo.Arg(xo.Int8, pointer=True, name="io_buffer"),
+                    xo.Arg(xo.UInt64, name="track_flags"),
                 ],
             )
         }
@@ -770,7 +774,7 @@ class Tracker:
 
         if ele_stop is not None:
             assert ele_stop >= 0
-            assert ele_stop < self.num_elements
+            assert ele_stop <= self.num_elements
 
         assert num_turns >= 1
 
@@ -842,7 +846,7 @@ class Tracker:
                 ret = pp.track(particles)
             else:
                 # The start part is a non-collective tracker
-                if (ele_stop is not None
+                if (ele_stop is not None and ele_stop < self.num_elements
                     and tt == num_turns - 1 and self._element_part[ele_stop] == ipp):
                     # The stop element is also in this part, so track until ele_stop
                     i_stop_in_part = self._element_index_in_part[ele_stop]
@@ -852,7 +856,7 @@ class Tracker:
                     # Track until end of part
                     ret = pp.track(particles, ele_start=i_start_in_part, turn_by_turn_monitor=monitor)
 
-        elif (ele_stop is not None
+        elif (ele_stop is not None and ele_stop < self.num_elements
                 and tt == num_turns-1 and self._element_part[ele_stop] == ipp):
             # We are in the part that contains the stop element
             i_stop_in_part = self._element_index_in_part[ele_stop]
@@ -867,6 +871,11 @@ class Tracker:
             if isinstance(pp, TrackerPartNonCollective):
                 ret = pp.track(particles, turn_by_turn_monitor=monitor)
             else:
+                if hasattr(monitor, 'ebe_mode') and monitor.ebe_mode == 1:
+                    assert monitor._context is particles._context, (
+                        'Element-by-element monitor not supported in multi-context'
+                        ' mode')
+                    monitor.track(particles)
                 ret = pp.track(particles)
 
         return stop_tracking, skip, ret
@@ -875,7 +884,8 @@ class Tracker:
         """
         Resume a track session that had been placed on hold.
         """
-        return self._track_with_collective(particles=None, _session_to_resume=session)
+        return self._track_with_collective(particles=None, _session_to_resume=session,
+                                           _reset_log=False)
 
     def _track_with_collective(
         self,
@@ -904,17 +914,12 @@ class Tracker:
                                       ' tracking')
 
         if log is not None:
-            if isinstance(log, str):
-                log = [log]
             if isinstance(log, (list, tuple)):
                 log = Log(*log)
+            elif not isinstance(log, Log):
+                log = Log(log)
         if log is not None and _reset_log:
-            if self.line.enable_time_dependent_vars:
-                self.line.log_last_track = {kk: [] for kk in log}
-            else:
-                raise NotImplementedError(
-                    'log can be used only when time-dependent variables are '
-                    'enabled in the line')
+            self.line.log_last_track = {}
 
         self._check_invalidated()
 
@@ -944,6 +949,7 @@ class Tracker:
                                     '_context_needs_clean_active_lost_state']
             tt_resume = _session_to_resume['tt']
             ipp_resume = _session_to_resume['ipp']
+            log = _session_to_resume['log']
             _session_to_resume['resumed'] = True
         else:
             (ele_start, ele_stop, num_turns, flag_monitor, monitor,
@@ -959,54 +965,17 @@ class Tracker:
             if tt_resume is not None and tt < tt_resume:
                 continue
 
-            if (flag_monitor and (ele_start == 0 or tt>0)): # second condition is for delayed start
-                if not(tt_resume is not None and tt == tt_resume):
+            if (flag_monitor and (ele_start == 0 or tt>0) # second condition is for delayed start
+                and not _is_resume_within_turn(tt, tt_resume)):
                     monitor.track(particles)
 
             # Time dependent vars and energy ramping
             if self.line.enable_time_dependent_vars:
-                # Find first active particle
-                state = particles.state
-                if isinstance(particles._context, xo.ContextPyopencl):
-                    state = state.get()
-                ii_first_active = int((state > 0).argmax())
-                if ii_first_active == 0 and particles._xobject.state[0] <= 0:
-                    # No active particles
-                    at_turn = 0 # convenient for multi-turn injection
-                else:
-                    at_turn = particles._xobject.at_turn[ii_first_active]
+                self._handle_time_dependent_vars(particles=particles)
 
-                if self.line.energy_program is not None:
-                    t_turn = self.line.energy_program.get_t_s_at_turn(at_turn)
-                else:
-                    beta0 = particles._xobject.beta0[ii_first_active]
-                    t_turn = (at_turn * self._tracker_data_base.line_length
-                            / (beta0 * clight))
-
-                # Clean leftover from previous trackings
-                if (self.line._t_last_update_time_dependent_vars and
-                   t_turn < self.line._t_last_update_time_dependent_vars):
-                    self.line._t_last_update_time_dependent_vars = None
-
-                if (self.line._t_last_update_time_dependent_vars is None
-                    or self.line.dt_update_time_dependent_vars is None
-                    or t_turn > self.line._t_last_update_time_dependent_vars
-                                + self.line.dt_update_time_dependent_vars):
-                    self.line._t_last_update_time_dependent_vars = t_turn
-                    self.vars['t_turn_s'] = t_turn
-
-                    if self.line.energy_program is not None:
-                        p0c = self.line.particle_ref._xobject.p0c[0]
-                        particles.update_p0c_and_energy_deviations(p0c)
-
-                if log is not None:
-                    for kk in log:
-                        if log[kk] == None:
-                            self.line.log_last_track[kk].append(self.line.vv[kk])
-                        else:
-                            ff = log[kk]
-                            self.line.log_last_track[kk].append(
-                                                    ff(self.line, particles))
+            if log is not None and not _is_resume_within_turn(tt, tt_resume):
+                self._handle_log(_session_to_resume=_session_to_resume,
+                                 particles=particles, log=log)
 
             moveback_to_buffer = None
             moveback_to_offset = None
@@ -1066,7 +1035,8 @@ class Tracker:
                             'moveback_to_offset': moveback_to_offset,
                             'ipp': ipp,
                             'tt': tt,
-                            'resumed': False
+                            'resumed': False,
+                            'log': log
                         }
                     return PipelineStatus(on_hold=True, data=session_on_hold)
 
@@ -1078,7 +1048,8 @@ class Tracker:
                 if not isinstance(pp, TrackerPartNonCollective) and not stop_tracking:
                     if moveback_to_buffer is not None: # The particles object is temporarily on CPU
                         if not hasattr(self, '_zerodrift_cpu'):
-                            self._zerodrift_cpu = self._zerodrift.copy(particles._buffer.context)
+                            self._zerodrift_cpu = self._zerodrift.copy(
+                                _context=particles._buffer.context)
                         self._zerodrift_cpu.track(particles, increment_at_element=True)
                     else:
                         self._zerodrift.track(particles, increment_at_element=True)
@@ -1103,6 +1074,10 @@ class Tracker:
                     particles._num_active_particles = -1
                     particles._num_lost_particles = -1
 
+            if monitor is not None and monitor.ebe_mode == 1 and (
+                ele_stop is None or ele_stop == self.num_elements):
+                monitor.track(particles)
+
             # Increment at_turn and reset at_element
             # (use the non-collective track method to perform only end-turn actions)
             self._track_no_collective(particles,
@@ -1111,7 +1086,6 @@ class Tracker:
 
         self.record_last_track = monitor
 
-
     def _track_no_collective(
         self,
         particles,
@@ -1144,8 +1118,8 @@ class Tracker:
                 raise ValueError("This line is not backtrackable.")
             kwargs.pop('self')
             kwargs.pop('backtrack')
-            with xt.line._preserve_config(self):
-                self.config.XSUITE_BACKTRACK = True
+            with xt.line._preserve_track_flags(self.line):
+                self.track_flags.XS_FLAG_BACKTRACK = True
                 return self._track_no_collective(**kwargs)
 
         self.local_particle_src = particles.gen_local_particle_api()
@@ -1181,6 +1155,8 @@ class Tracker:
         assert ele_start >= 0
         assert ele_start <= self.num_elements
 
+        track_flags = self.track_flags.get_flags_register()
+
         # Logic to split the tracking turns:
         # Case 1: 0 <= start < stop <= L
         #      Track first turn from start until stop (with num_elements_first_turn=stop-start)
@@ -1288,6 +1264,7 @@ class Tracker:
             buffer_tbt_monitor=buffer_monitor,
             offset_tbt_monitor=offset_monitor,
             io_buffer=self.io_buffer.buffer,
+            track_flags=track_flags
         )
 
         # Middle turns
@@ -1308,6 +1285,7 @@ class Tracker:
                 buffer_tbt_monitor=buffer_monitor,
                 offset_tbt_monitor=offset_monitor,
                 io_buffer=self.io_buffer.buffer,
+                track_flags=track_flags
             )
 
         # Last turn, only if incomplete
@@ -1328,6 +1306,7 @@ class Tracker:
                 buffer_tbt_monitor=buffer_monitor,
                 offset_tbt_monitor=offset_monitor,
                 io_buffer=self.io_buffer.buffer,
+                track_flags=track_flags
             )
 
         self.record_last_track = monitor
@@ -1475,6 +1454,65 @@ class Tracker:
             line_element_classes=self.line_element_classes,
             verbose=True)
 
+    def _handle_time_dependent_vars(self, particles):
+
+        # Find first active particle
+        state = particles.state
+        if isinstance(particles._context, xo.ContextPyopencl):
+            state = state.get()
+        ii_first_active = int((state > 0).argmax())
+        if ii_first_active == 0 and particles._xobject.state[0] <= 0:
+            # No active particles
+            at_turn = 0 # convenient for multi-turn injection
+        else:
+            at_turn = particles._xobject.at_turn[ii_first_active]
+
+        if self.line.energy_program is not None:
+            t_turn = self.line.energy_program.get_t_s_at_turn(at_turn)
+        else:
+            beta0 = particles._xobject.beta0[ii_first_active]
+            t_turn = (at_turn * self._tracker_data_base.line_length
+                    / (beta0 * clight))
+
+        # Clean leftover from previous trackings
+        if (self.line._t_last_update_time_dependent_vars and
+            t_turn < self.line._t_last_update_time_dependent_vars):
+            self.line._t_last_update_time_dependent_vars = None
+
+        if (self.line._t_last_update_time_dependent_vars is None
+            or self.line.dt_update_time_dependent_vars is None
+            or t_turn > self.line._t_last_update_time_dependent_vars
+                        + self.line.dt_update_time_dependent_vars):
+            self.line._t_last_update_time_dependent_vars = t_turn
+            self.vars['t_turn_s'] = t_turn
+
+            if self.line.energy_program is not None:
+                p0c = self.line.particle_ref._xobject.p0c[0]
+                particles.update_p0c_and_energy_deviations(p0c, update_pxpy=True)
+
+    def _handle_log(self, _session_to_resume, particles, log):
+        if _session_to_resume is not None:
+            plog = _session_to_resume['particles']
+        else:
+            plog = particles
+        for kk in log:
+            if log[kk] == None:
+                if kk not in self.line.log_last_track:
+                    self.line.log_last_track[kk] = []
+                self.line.log_last_track[kk].append(self.line.vv[kk])
+            else:
+                ff = log[kk]
+                val = ff(self.line, plog)
+                if hasattr(ff, '_store'):
+                    for nn in ff._store:
+                        if nn not in self.line.log_last_track:
+                            self.line.log_last_track[nn] = []
+                        self.line.log_last_track[nn].append(val[nn])
+                else:
+                    if kk not in self.line.log_last_track:
+                        self.line.log_last_track[kk] = []
+                    self.line.log_last_track[kk].append(val)
+
 
 class TrackerConfig(UserDict):
 
@@ -1574,5 +1612,13 @@ class Log(dict):
     def __init__(self, *args, **kwargs):
         self.__dict__ = self
         self.update(kwargs)
+        unnamed_indx = 0
         for arg in args:
-            self[arg] = None
\ No newline at end of file
+            if isinstance(arg, str):
+                self[arg] = None
+            else:
+                self[f'_unnamed_{unnamed_indx}'] = arg
+                unnamed_indx += 1
+
+def _is_resume_within_turn(tt, tt_resume):
+    return tt_resume is not None and tt == tt_resume
\ No newline at end of file
